@@ -72,6 +72,7 @@ class Redis
       "exists?"          => [ :all ],
       "expire"           => [ :first ],
       "expireat"         => [ :first ],
+      "expiretime"       => [ :first ],
       "eval"             => [ :eval_style ],
       "evalsha"          => [ :eval_style ],
       "get"              => [ :first ],
@@ -122,6 +123,7 @@ class Redis
       "persist"          => [ :first ],
       "pexpire"          => [ :first ],
       "pexpireat"        => [ :first ],
+      "pexpiretime"      => [ :first ],
       "pfadd"            => [ :first ],
       "pfcount"          => [ :all ],
       "pfmerge"          => [ :all ],
@@ -138,6 +140,7 @@ class Redis
       "rpush"            => [ :first ],
       "rpushx"           => [ :first ],
       "sadd"             => [ :first ],
+      "sadd?"            => [ :first ],
       "scard"            => [ :first ],
       "scan"             => [ :scan_style, :second ],
       "scan_each"        => [ :scan_style, :all ],
@@ -158,6 +161,7 @@ class Redis
       "spop"             => [ :first ],
       "srandmember"      => [ :first ],
       "srem"             => [ :first ],
+      "srem?"            => [ :first ],
       "sscan"            => [ :first ],
       "sscan_each"       => [ :first ],
       "strlen"           => [ :first ],
@@ -202,6 +206,7 @@ class Redis
     HELPER_COMMANDS = {
       "auth"             => [],
       "disconnect!"      => [],
+      "close"            => [],
       "echo"             => [],
       "ping"             => [],
       "time"             => [],
@@ -238,6 +243,16 @@ class Redis
     # Support 1.8.7 by providing a namespaced reference to Enumerable::Enumerator
     Enumerator = Enumerable::Enumerator unless defined?(::Enumerator)
 
+    # This is used by the Redis gem to determine whether or not to display that deprecation message.
+    @sadd_returns_boolean = true
+
+    # This is used by the Redis gem to determine whether or not to display that deprecation message.
+    @srem_returns_boolean = true
+
+    class << self
+      attr_accessor :sadd_returns_boolean, :srem_returns_boolean
+    end
+
     attr_writer :namespace
     attr_reader :redis
     attr_accessor :warning
@@ -329,6 +344,32 @@ class Redis
     end
     ruby2_keywords(:eval) if respond_to?(:ruby2_keywords, true)
 
+    # This operation can run for a very long time if the namespace contains lots of keys!
+    # It should be used in tests, or when the namespace is small enough
+    # and you are sure you know what you are doing.
+    def clear
+      if warning?
+        warn("This operation can run for a very long time if the namespace contains lots of keys! " +
+             "It should be used in tests, or when the namespace is small enough " +
+             "and you are sure you know what you are doing.")
+      end
+
+      batch_size = 1000
+
+      if supports_scan?
+        cursor = "0"
+        begin
+          cursor, keys = scan(cursor, count: batch_size)
+          del(*keys) unless keys.empty?
+        end until cursor == "0"
+      else
+        all_keys = keys("*")
+        all_keys.each_slice(batch_size) do |keys|
+          del(*keys)
+        end
+      end
+    end
+
     ADMINISTRATIVE_COMMANDS.keys.each do |command|
       define_method(command) do |*args, &block|
         raise NoMethodError if deprecations?
@@ -371,7 +412,8 @@ class Redis
                "passthrough has been deprecated and will be removed in " +
                "redis-namespace 2.0 (at #{call_site})")
         end
-        @redis.send(command, *args, &block)
+
+        wrapped_send(@redis, command, args, &block)
       else
         super
       end
@@ -476,7 +518,7 @@ class Redis
       end
 
       # Dispatch the command to Redis and store the result.
-      result = @redis.send(command, *args, &block)
+      result = wrapped_send(@redis, command, args, &block)
 
       # Don't try to remove namespace from a Redis::Future, you can't.
       return result if result.is_a?(Redis::Future)
@@ -513,6 +555,16 @@ class Redis
       end
     end
 
+    def wrapped_send(redis_client, command, args = [], &block)
+      if redis_client.class.name == "ConnectionPool"
+        redis_client.with do |pool_connection|
+          pool_connection.send(command, *args, &block)
+        end
+      else
+        redis_client.send(command, *args, &block)
+      end
+    end
+
     # Avoid modifying the caller's (pass-by-reference) arguments.
     def clone_args(arg)
       if arg.is_a?(Array)
@@ -530,13 +582,10 @@ class Redis
 
     def namespaced_block(command, &block)
       if block.arity == 0
-        redis.send(command, &block)
+        wrapped_send(redis, command, &block)
       else
-        redis.send(command) do |r|
-          copy = dup
-          copy.redis = r
-          yield copy
-        end
+        outer_block = proc { |r| copy = dup; copy.redis = r; yield copy }
+        wrapped_send(redis, command, &outer_block)
       end
     end
 
@@ -582,5 +631,10 @@ class Redis
         Enumerator.new(&block)
       end
     end
+
+    def supports_scan?
+      redis_version = @redis.info["redis_version"]
+      Gem::Version.new(redis_version) >= Gem::Version.new("2.8.0")
+    end
   end
 end
