@@ -1,6 +1,7 @@
 # encoding: utf-8
 
 require File.dirname(__FILE__) + '/spec_helper'
+require 'connection_pool'
 
 describe "redis" do
   @redis_version = Gem::Version.new(Redis.new.info["redis_version"])
@@ -184,6 +185,24 @@ describe "redis" do
     expect(@namespaced.mapped_mget('foo', 'baz', 'bar')).to eq({'foo'=>'1000', 'bar'=>'2000', 'baz' => nil})
   end
 
+  it "should utilize connection_pool while using a namespace with mget" do
+    memo = @namespaced
+    connection_pool = ConnectionPool.new(size: 2, timeout: 2) { Redis.new db: 15 }
+    @namespaced = Redis::Namespace.new(:ns, redis: connection_pool)
+
+    expect(connection_pool).to receive(:with).and_call_original do |arg|
+      expect(arg).to be(an_instance_of(Redis))
+    end.at_least(:once)
+
+    @namespaced.set('foo', 1000)
+    @namespaced.set('bar', 2000)
+    expect(@namespaced.mapped_mget('foo', 'bar')).to eq({ 'foo' => '1000', 'bar' => '2000' })
+    expect(@namespaced.mapped_mget('foo', 'baz', 'bar')).to eq({'foo'=>'1000', 'bar'=>'2000', 'baz' => nil})
+    @redis.get('foo').should eq('bar')
+
+    @namespaced = memo
+  end
+
   it "should be able to use a namespace with mset" do
     @namespaced.mset('foo', '1000', 'bar', '2000')
     expect(@namespaced.mapped_mget('foo', 'bar')).to eq({ 'foo' => '1000', 'bar' => '2000' })
@@ -339,6 +358,17 @@ describe "redis" do
     expect(values).to match_array(['banana', 'eggplant'])
   end
 
+  it "should add a new member" do
+    expect(@namespaced.sadd?('foo', 1)).to eq(true)
+    expect(@namespaced.sadd?('foo', 1)).to eq(false)
+  end
+
+  it "should remove members" do
+    @namespaced.sadd('foo', 1)
+    expect(@namespaced.srem?('foo', 1)).to eq(true)
+    expect(@namespaced.srem?('foo', 1)).to eq(false)
+  end
+
   it "should add namespace to sort" do
     @namespaced.sadd('foo', 1)
     @namespaced.sadd('foo', 2)
@@ -376,6 +406,26 @@ describe "redis" do
     expect(@namespaced.hgetall("foo")).to eq({"key1" => "value1"})
   end
 
+  it "should utilize connection_pool while adding namepsace to multi blocks" do
+    memo = @namespaced
+    connection_pool = ConnectionPool.new(size: 2, timeout: 2) { Redis.new db: 15 }
+    @namespaced = Redis::Namespace.new(:ns, redis: connection_pool)
+
+    expect(connection_pool).to receive(:with).and_call_original do |arg|
+      expect(arg).to be(an_instance_of(Redis))
+    end.at_least(:once)
+
+    @namespaced.mapped_hmset "foo", {"key" => "value"}
+    @namespaced.multi do |r|
+      r.del "foo"
+      r.mapped_hmset "foo", {"key1" => "value1"}
+    end
+    expect(@redis.get("foo")).to eq("bar")
+    expect(@namespaced.hgetall("foo")).to eq({"key1" => "value1"})
+
+    @namespaced = memo
+  end
+
   it "should pass through multi commands without block" do
     @namespaced.mapped_hmset "foo", {"key" => "value"}
 
@@ -387,6 +437,28 @@ describe "redis" do
     expect(@namespaced.hgetall("foo")).to eq({"key1" => "value1"})
   end
 
+  it "should utilize connection_pool while passing through multi commands without block" do
+    memo = @namespaced
+    connection_pool = ConnectionPool.new(size: 2, timeout: 2) { Redis.new db: 15 }
+    @namespaced = Redis::Namespace.new(:ns, redis: connection_pool)
+
+    expect(connection_pool).to receive(:with).and_call_original do |arg|
+      expect(arg).to be(an_instance_of(Redis))
+    end.at_least(:once)
+
+    @namespaced.mapped_hmset "foo", {"key" => "value"}
+
+    @namespaced.multi
+    @namespaced.del "foo"
+    @namespaced.mapped_hmset "foo", {"key1" => "value1"}
+    @namespaced.exec
+
+    expect(@namespaced.hgetall("foo")).to eq({"key1" => "value1"})
+    expect(@redis.get("foo")).to eq("bar")
+
+    @namespaced = memo
+  end
+
   it 'should return futures without attempting to remove namespaces' do
     @namespaced.multi do
       @future = @namespaced.keys('*')
@@ -403,6 +475,26 @@ describe "redis" do
     expect(@namespaced.hgetall("foo")).to eq({"key1" => "value1"})
   end
 
+  it "should utilize connection_pool while adding namespace to pipelined blocks" do
+    memo = @namespaced
+    connection_pool = ConnectionPool.new(size: 2, timeout: 2) { Redis.new db: 15 }
+    @namespaced = Redis::Namespace.new(:ns, redis: connection_pool)
+
+    expect(connection_pool).to receive(:with).and_call_original do |arg|
+      expect(arg).to be(an_instance_of(Redis))
+    end.at_least(:once)
+
+    @namespaced.mapped_hmset "foo", {"key" => "value"}
+    @namespaced.pipelined do |r|
+      r.del "foo"
+      r.mapped_hmset "foo", {"key1" => "value1"}
+    end
+    expect(@namespaced.hgetall("foo")).to eq({"key1" => "value1"})
+    expect(@redis.get("foo")).to eq("bar")
+
+    @namespaced = memo
+  end
+
   it "should returned response array from pipelined block" do
     @namespaced.mset "foo", "bar", "key", "value"
     result = @namespaced.pipelined do |r|
@@ -574,6 +666,12 @@ describe "redis" do
         expect(@redis.ttl("ns:foo")).to satisfy {|v| (0..1).include?(v) }
       end
 
+      it "should namespace expiretime" do
+        @namespaced.set('mykey', 'Hello')
+        @namespaced.expireat('mykey', 2000000000)
+        expect(@namespaced.expiretime('mykey')).to eq(2000000000)
+      end
+
       it "should namespace hincrbyfloat" do
         @namespaced.hset('mykey', 'field', 10.50)
         expect(@namespaced.hincrbyfloat('mykey', 'field', 0.1)).to eq(10.6)
@@ -606,6 +704,12 @@ describe "redis" do
         expect(@namespaced.pexpire('mykey', 1555555555005)).to eq(true)
       end
 
+      it "should namespace pexpiretime" do
+        @namespaced.set('mykey', 'Hello')
+        @namespaced.pexpireat('mykey', 2000000000000)
+        expect(@namespaced.pexpiretime('mykey')).to eq(2000000000000)
+      end
+
       it "should namespace psetex" do
         expect(@namespaced.psetex('mykey', 10000, 'Hello')).to eq('OK')
         expect(@namespaced.get('mykey')).to eq('Hello')
@@ -967,4 +1071,30 @@ describe "redis" do
       expect(sub_sub_namespaced.full_namespace).to eql("ns:sub1:sub2")
     end
   end
+
+  describe :clear do
+    it "warns with helpful output" do
+      expect { @namespaced.clear }.to output(/can run for a very long time/).to_stderr
+    end
+
+    it "should delete all the keys" do
+      @redis.set("foo", "bar")
+      @namespaced.mset("foo1", "bar", "foo2", "bar")
+      capture_stderr { @namespaced.clear }
+
+      expect(@redis.keys).to eq ["foo"]
+      expect(@namespaced.keys).to be_empty
+    end
+
+    it "should delete all the keys in older redis" do
+      allow(@redis).to receive(:info).and_return({ "redis_version" => "2.7.0" })
+
+      @redis.set("foo", "bar")
+      @namespaced.mset("foo1", "bar", "foo2", "bar")
+      capture_stderr { @namespaced.clear }
+
+      expect(@redis.keys).to eq ["foo"]
+      expect(@namespaced.keys).to be_empty
+    end
+  end
 end
